# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import field_mask_pb2  # type: ignore
import proto  # type: ignore

from google.shopping.merchant_datasources_v1.types import datasourcetypes, fileinputs

__protobuf__ = proto.module(
    package="google.shopping.merchant.datasources.v1",
    manifest={
        "DataSource",
        "GetDataSourceRequest",
        "ListDataSourcesRequest",
        "ListDataSourcesResponse",
        "CreateDataSourceRequest",
        "UpdateDataSourceRequest",
        "FetchDataSourceRequest",
        "DeleteDataSourceRequest",
    },
)


class DataSource(proto.Message):
    r"""The `data source </merchant/api/guides/data-sources/overview>`__ for
    the Merchant Center account.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        primary_product_data_source (google.shopping.merchant_datasources_v1.types.PrimaryProductDataSource):
            The `primary data
            source <https://support.google.com/merchants/answer/7439058>`__
            for local and online products.

            This field is a member of `oneof`_ ``Type``.
        supplemental_product_data_source (google.shopping.merchant_datasources_v1.types.SupplementalProductDataSource):
            The `supplemental data
            source <https://support.google.com/merchants/answer/7439058>`__
            for local and online products.

            This field is a member of `oneof`_ ``Type``.
        local_inventory_data_source (google.shopping.merchant_datasources_v1.types.LocalInventoryDataSource):
            The `local
            inventory <https://support.google.com/merchants/answer/7023001>`__
            data source.

            This field is a member of `oneof`_ ``Type``.
        regional_inventory_data_source (google.shopping.merchant_datasources_v1.types.RegionalInventoryDataSource):
            The `regional
            inventory <https://support.google.com/merchants/answer/7439058>`__
            data source.

            This field is a member of `oneof`_ ``Type``.
        promotion_data_source (google.shopping.merchant_datasources_v1.types.PromotionDataSource):
            The
            `promotion <https://support.google.com/merchants/answer/2906014>`__
            data source.

            This field is a member of `oneof`_ ``Type``.
        product_review_data_source (google.shopping.merchant_datasources_v1.types.ProductReviewDataSource):
            The `product
            review <https://support.google.com/merchants/answer/7045996>`__
            data source.

            This field is a member of `oneof`_ ``Type``.
        merchant_review_data_source (google.shopping.merchant_datasources_v1.types.MerchantReviewDataSource):
            The `merchant
            review <https://support.google.com/merchants/answer/7045996>`__
            data source.

            This field is a member of `oneof`_ ``Type``.
        name (str):
            Required. Identifier. The name of the data source. Format:
            ``accounts/{account}/dataSources/{datasource}``
        data_source_id (int):
            Output only. The data source id.
        display_name (str):
            Required. The displayed data source name in
            the Merchant Center UI.
        input (google.shopping.merchant_datasources_v1.types.DataSource.Input):
            Output only. Determines the type of input to
            the data source. Based on the input some
            settings might not work. Only generic data
            sources can be created through the API.
        file_input (google.shopping.merchant_datasources_v1.types.FileInput):
            Optional. The field is used only when data is
            managed through a file.
    """

    class Input(proto.Enum):
        r"""Determines the type of input to the data source. Based on the
        input some settings might not be supported.

        Values:
            INPUT_UNSPECIFIED (0):
                Input unspecified.
            API (1):
                Represents data sources for which the data is
                primarily provided through the API.
            FILE (2):
                Represents data sources for which the data is
                primarily provided through file input. Data can
                still be provided through the API.
            UI (3):
                The data source for products added directly
                in Merchant Center.
                This type of data source can not be created or
                updated through this API, only by Merchant
                Center UI.

                This type of data source is read only.
            AUTOFEED (4):
                This is also known as `Automated
                feeds <https://support.google.com/merchants/answer/12158480>`__
                used to automatically build your product data. This type of
                data source can be enabled or disabled through the Accounts
                sub-API.
        """
        INPUT_UNSPECIFIED = 0
        API = 1
        FILE = 2
        UI = 3
        AUTOFEED = 4

    primary_product_data_source: datasourcetypes.PrimaryProductDataSource = proto.Field(
        proto.MESSAGE,
        number=4,
        oneof="Type",
        message=datasourcetypes.PrimaryProductDataSource,
    )
    supplemental_product_data_source: datasourcetypes.SupplementalProductDataSource = (
        proto.Field(
            proto.MESSAGE,
            number=5,
            oneof="Type",
            message=datasourcetypes.SupplementalProductDataSource,
        )
    )
    local_inventory_data_source: datasourcetypes.LocalInventoryDataSource = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="Type",
        message=datasourcetypes.LocalInventoryDataSource,
    )
    regional_inventory_data_source: datasourcetypes.RegionalInventoryDataSource = (
        proto.Field(
            proto.MESSAGE,
            number=7,
            oneof="Type",
            message=datasourcetypes.RegionalInventoryDataSource,
        )
    )
    promotion_data_source: datasourcetypes.PromotionDataSource = proto.Field(
        proto.MESSAGE,
        number=8,
        oneof="Type",
        message=datasourcetypes.PromotionDataSource,
    )
    product_review_data_source: datasourcetypes.ProductReviewDataSource = proto.Field(
        proto.MESSAGE,
        number=9,
        oneof="Type",
        message=datasourcetypes.ProductReviewDataSource,
    )
    merchant_review_data_source: datasourcetypes.MerchantReviewDataSource = proto.Field(
        proto.MESSAGE,
        number=12,
        oneof="Type",
        message=datasourcetypes.MerchantReviewDataSource,
    )
    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    data_source_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    input: Input = proto.Field(
        proto.ENUM,
        number=10,
        enum=Input,
    )
    file_input: fileinputs.FileInput = proto.Field(
        proto.MESSAGE,
        number=11,
        message=fileinputs.FileInput,
    )


class GetDataSourceRequest(proto.Message):
    r"""Request message for the GetDataSource method.

    Attributes:
        name (str):
            Required. The name of the data source to retrieve. Format:
            ``accounts/{account}/dataSources/{datasource}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListDataSourcesRequest(proto.Message):
    r"""Request message for the ListDataSources method.

    Attributes:
        parent (str):
            Required. The account to list data sources for. Format:
            ``accounts/{account}``
        page_size (int):
            Optional. The maximum number of data sources
            to return. The service may return fewer than
            this value. The maximum value is 1000; values
            above 1000 will be coerced to 1000. If
            unspecified, the maximum number of data sources
            will be returned.
        page_token (str):
            Optional. A page token, received from a previous
            ``ListDataSources`` call. Provide this to retrieve the
            subsequent page.

            When paginating, all other parameters provided to
            ``ListDataSources`` must match the call that provided the
            page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListDataSourcesResponse(proto.Message):
    r"""Response message for the ListDataSources method.

    Attributes:
        data_sources (MutableSequence[google.shopping.merchant_datasources_v1.types.DataSource]):
            The data sources from the specified account.
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    data_sources: MutableSequence["DataSource"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="DataSource",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateDataSourceRequest(proto.Message):
    r"""Request message for the CreateDataSource method.

    Attributes:
        parent (str):
            Required. The account where this data source will be
            created. Format: ``accounts/{account}``
        data_source (google.shopping.merchant_datasources_v1.types.DataSource):
            Required. The data source to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    data_source: "DataSource" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="DataSource",
    )


class UpdateDataSourceRequest(proto.Message):
    r"""Request message for the UpdateDataSource method.

    Attributes:
        data_source (google.shopping.merchant_datasources_v1.types.DataSource):
            Required. The data source resource to update.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The list of data source fields to be updated.

            Fields specified in the update mask without a value
            specified in the body will be deleted from the data source.

            Providing special "\*" value for full data source
            replacement is not supported.

            For example, If you insert ``updateMask=displayName`` in the
            request, it will only update the ``displayName`` leaving all
            other fields untouched.
    """

    data_source: "DataSource" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="DataSource",
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class FetchDataSourceRequest(proto.Message):
    r"""Request message for the FetchDataSource method.

    Attributes:
        name (str):
            Required. The name of the data source resource to fetch.
            Format: ``accounts/{account}/dataSources/{datasource}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class DeleteDataSourceRequest(proto.Message):
    r"""Request message for the DeleteDataSource method.

    Attributes:
        name (str):
            Required. The name of the data source to delete. Format:
            ``accounts/{account}/dataSources/{datasource}``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.shopping.type.types import types
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.shopping.merchant.datasources.v1",
    manifest={
        "PrimaryProductDataSource",
        "SupplementalProductDataSource",
        "LocalInventoryDataSource",
        "RegionalInventoryDataSource",
        "PromotionDataSource",
        "ProductReviewDataSource",
        "MerchantReviewDataSource",
        "DataSourceReference",
    },
)


class PrimaryProductDataSource(proto.Message):
    r"""The primary data source for local and online products.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        legacy_local (bool):
            Optional. Immutable. Determines whether the products of this
            data source are **only** targeting local destinations.
            Legacy local products are prefixed with ``local~`` in the
            product resource ID. For example,
            ``accounts/123/products/local~en~US~sku123``.
        feed_label (str):
            Optional. Immutable. The feed label that is specified on the
            data source level.

            Must be less than or equal to 20 uppercase letters (A-Z),
            numbers (0-9), and dashes (-).

            See also `migration to feed
            labels <https://developers.google.com/shopping-content/guides/products/feed-labels>`__.

            ``feedLabel`` and ``contentLanguage`` must be either both
            set or unset for data sources with product content type.
            They must be set for data sources with a file input.

            If set, the data source will only accept products matching
            this combination. If unset, the data source will accept
            products without that restriction.

            This field is a member of `oneof`_ ``_feed_label``.
        content_language (str):
            Optional. Immutable. The two-letter ISO 639-1 language of
            the items in the data source.

            ``feedLabel`` and ``contentLanguage`` must be either both
            set or unset. The fields can only be unset for data sources
            without file input.

            If set, the data source will only accept products matching
            this combination. If unset, the data source will accept
            products without that restriction.

            This field is a member of `oneof`_ ``_content_language``.
        countries (MutableSequence[str]):
            Optional. The countries where the items may be displayed.
            Represented as a `CLDR territory
            code <https://github.com/unicode-org/cldr/blob/latest/common/main/en.xml>`__.
        default_rule (google.shopping.merchant_datasources_v1.types.PrimaryProductDataSource.DefaultRule):
            Optional. Default rule management of the data
            source. If set, the linked data sources will be
            replaced.
        destinations (MutableSequence[google.shopping.merchant_datasources_v1.types.PrimaryProductDataSource.Destination]):
            Optional. A list of destinations describing
            where products of the data source can be shown.

            When retrieving the data source, the list
            contains all the destinations that can be used
            for the data source, including the ones that are
            disabled for the data source but enabled for the
            account.

            Only destinations that are enabled on the
            account, for example through program
            participation, can be enabled on the data
            source.

            If unset, during creation, the destinations will
            be inherited based on the account level program
            participation.

            If set, during creation or update, the data
            source will be set only for the specified
            destinations.

            Updating this field requires at least one
            destination.
    """

    class DefaultRule(proto.Message):
        r"""Default rule management of the data source.

        Attributes:
            take_from_data_sources (MutableSequence[google.shopping.merchant_datasources_v1.types.DataSourceReference]):
                Required. The list of data sources linked in the `default
                rule <https://support.google.com/merchants/answer/7450276>`__.
                This list is ordered by the default rule priority of joining
                the data. It might include none or multiple references to
                ``self`` and supplemental data sources.

                The list must not be empty.

                To link the data source to the default rule, you need to add
                a new reference to this list (in sequential order).

                To unlink the data source from the default rule, you need to
                remove the given reference from this list.

                Changing the order of this list will result in changing the
                priority of data sources in the default rule.

                For example, providing the following list: [``1001``,
                ``self``] will take attribute values from supplemental data
                source ``1001``, and fallback to ``self`` if the attribute
                is not set in ``1001``.
        """

        take_from_data_sources: MutableSequence[
            "DataSourceReference"
        ] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="DataSourceReference",
        )

    class Destination(proto.Message):
        r"""Destinations also known as `Marketing
        methods <https://support.google.com/merchants/answer/15130232>`__
        selections.

        Attributes:
            destination (google.shopping.type.types.Destination.DestinationEnum):
                `Marketing
                methods <https://support.google.com/merchants/answer/15130232>`__
                (also known as destination) selections.
            state (google.shopping.merchant_datasources_v1.types.PrimaryProductDataSource.Destination.State):
                The state of the destination.
        """

        class State(proto.Enum):
            r"""The state of the destination.

            Values:
                STATE_UNSPECIFIED (0):
                    Not specified.
                ENABLED (1):
                    Indicates that the destination is enabled.
                DISABLED (2):
                    Indicates that the destination is disabled.
            """
            STATE_UNSPECIFIED = 0
            ENABLED = 1
            DISABLED = 2

        destination: types.Destination.DestinationEnum = proto.Field(
            proto.ENUM,
            number=1,
            enum=types.Destination.DestinationEnum,
        )
        state: "PrimaryProductDataSource.Destination.State" = proto.Field(
            proto.ENUM,
            number=2,
            enum="PrimaryProductDataSource.Destination.State",
        )

    legacy_local: bool = proto.Field(
        proto.BOOL,
        number=11,
    )
    feed_label: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    content_language: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    countries: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )
    default_rule: DefaultRule = proto.Field(
        proto.MESSAGE,
        number=7,
        message=DefaultRule,
    )
    destinations: MutableSequence[Destination] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=Destination,
    )


class SupplementalProductDataSource(proto.Message):
    r"""The supplemental data source for local and online products.
    After creation, you should make sure to link the supplemental
    product data source into one or more primary product data
    sources.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        feed_label (str):
            Optional. Immutable. The feed label that is specified on the
            data source level.

            Must be less than or equal to 20 uppercase letters (A-Z),
            numbers (0-9), and dashes (-).

            See also `migration to feed
            labels <https://developers.google.com/shopping-content/guides/products/feed-labels>`__.

            ``feedLabel`` and ``contentLanguage`` must be either both
            set or unset for data sources with product content type.

            They must be set for data sources with a [file
            input][google.shopping.merchant.datasources.v1.FileInput].
            The fields must be unset for data sources without [file
            input][google.shopping.merchant.datasources.v1.FileInput].

            If set, the data source will only accept products matching
            this combination. If unset, the data source will accept
            produts without that restriction.

            This field is a member of `oneof`_ ``_feed_label``.
        content_language (str):
            Optional. Immutable. The two-letter ISO 639-1 language of
            the items in the data source.

            ``feedLabel`` and ``contentLanguage`` must be either both
            set or unset. The fields can only be unset for data sources
            without file input.

            If set, the data source will only accept products matching
            this combination. If unset, the data source will accept
            produts without that restriction.

            This field is a member of `oneof`_ ``_content_language``.
        referencing_primary_data_sources (MutableSequence[google.shopping.merchant_datasources_v1.types.DataSourceReference]):
            Output only. The (unordered and deduplicated)
            list of all primary data sources linked to this
            data source in either default or custom rules.
            Supplemental data source cannot be deleted
            before all links are removed.
    """

    feed_label: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    content_language: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    referencing_primary_data_sources: MutableSequence[
        "DataSourceReference"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="DataSourceReference",
    )


class LocalInventoryDataSource(proto.Message):
    r"""The local inventory data source type is only available for
    file inputs and can't be used to create API local inventory data
    sources.

    Attributes:
        feed_label (str):
            Required. Immutable. The feed label of the offers to which
            the local inventory is provided.

            Must be less than or equal to 20 uppercase letters (A-Z),
            numbers (0-9), and dashes (-).

            See also `migration to feed
            labels <https://developers.google.com/shopping-content/guides/products/feed-labels>`__.
        content_language (str):
            Required. Immutable. The two-letter ISO 639-1
            language of the items to which the local
            inventory is provided.
    """

    feed_label: str = proto.Field(
        proto.STRING,
        number=4,
    )
    content_language: str = proto.Field(
        proto.STRING,
        number=5,
    )


class RegionalInventoryDataSource(proto.Message):
    r"""

    Attributes:
        feed_label (str):
            Required. Immutable. The feed label of the offers to which
            the regional inventory is provided.

            Must be less than or equal to 20 uppercase letters (A-Z),
            numbers (0-9), and dashes (-).

            See also `migration to feed
            labels <https://developers.google.com/shopping-content/guides/products/feed-labels>`__.
        content_language (str):
            Required. Immutable. The two-letter ISO 639-1
            language of the items to which the regional
            inventory is provided.
    """

    feed_label: str = proto.Field(
        proto.STRING,
        number=4,
    )
    content_language: str = proto.Field(
        proto.STRING,
        number=5,
    )


class PromotionDataSource(proto.Message):
    r"""The promotion data source.

    Attributes:
        target_country (str):
            Required. Immutable. The target country used as part of the
            unique identifier. Represented as a `CLDR territory
            code <https://github.com/unicode-org/cldr/blob/latest/common/main/en.xml>`__.

            Promotions are only available in selected
            `countries <https://support.google.com/merchants/answer/4588460>`__.
        content_language (str):
            Required. Immutable. The two-letter ISO 639-1
            language of the items in the data source.
    """

    target_country: str = proto.Field(
        proto.STRING,
        number=1,
    )
    content_language: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ProductReviewDataSource(proto.Message):
    r"""The product review data source."""


class MerchantReviewDataSource(proto.Message):
    r"""The merchant review data source."""


class DataSourceReference(proto.Message):
    r"""Data source reference can be used to manage related data
    sources within the data source service.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        self_ (bool):
            Self should be used to reference the primary
            data source itself.

            This field is a member of `oneof`_ ``data_source_id``.
        primary_data_source_name (str):
            Optional. The name of the primary data source. Format:
            ``accounts/{account}/dataSources/{datasource}``

            This field is a member of `oneof`_ ``data_source_id``.
        supplemental_data_source_name (str):
            Optional. The name of the supplemental data source. Format:
            ``accounts/{account}/dataSources/{datasource}``

            This field is a member of `oneof`_ ``data_source_id``.
    """

    self_: bool = proto.Field(
        proto.BOOL,
        number=1,
        oneof="data_source_id",
    )
    primary_data_source_name: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="data_source_id",
    )
    supplemental_data_source_name: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="data_source_id",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.protobuf import timestamp_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.shopping.merchant.datasources.v1beta",
    manifest={
        "FileUpload",
        "GetFileUploadRequest",
    },
)


class FileUpload(proto.Message):
    r"""The file upload of a specific data source, that is, the
    result of the retrieval of the data source at a certain
    timestamp computed asynchronously when the data source
    processing is finished. Only applicable to file data sources.

    Attributes:
        name (str):
            Identifier. The name of the data source file upload. Format:
            ``{datasource.name=accounts/{account}/dataSources/{datasource}/fileUploads/{fileupload}}``
        data_source_id (int):
            Output only. The data source id.
        processing_state (google.shopping.merchant_datasources_v1beta.types.FileUpload.ProcessingState):
            Output only. The processing state of the data
            source.
        issues (MutableSequence[google.shopping.merchant_datasources_v1beta.types.FileUpload.Issue]):
            Output only. The list of issues occurring in
            the data source.
        items_total (int):
            Output only. The number of items in the data
            source that were processed.
        items_created (int):
            Output only. The number of items in the data
            source that were created.
        items_updated (int):
            Output only. The number of items in the data
            source that were updated.
        upload_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. The date at which the file of
            the data source was uploaded.
    """

    class ProcessingState(proto.Enum):
        r"""The processing state of the data source.

        Values:
            PROCESSING_STATE_UNSPECIFIED (0):
                Processing state unspecified.
            FAILED (1):
                The data source could not be processed or all
                the items had errors.
            IN_PROGRESS (2):
                The data source is being processed.
            SUCCEEDED (3):
                The data source was processed successfully,
                though some items might have had errors.
        """
        PROCESSING_STATE_UNSPECIFIED = 0
        FAILED = 1
        IN_PROGRESS = 2
        SUCCEEDED = 3

    class Issue(proto.Message):
        r"""An error occurring in the data source, like "invalid price".

        Attributes:
            title (str):
                Output only. The title of the issue, for
                example, "Item too big".
            description (str):
                Output only. The error description, for
                example, "Your data source contains items which
                have too many attributes, or are too big. These
                items will be dropped".
            code (str):
                Output only. The code of the error, for example,
                "validation/invalid_value". Returns "?" if the code is
                unknown.
            count (int):
                Output only. The number of occurrences of the
                error in the file upload.
            severity (google.shopping.merchant_datasources_v1beta.types.FileUpload.Issue.Severity):
                Output only. The severity of the issue.
            documentation_uri (str):
                Output only. Link to the documentation
                explaining the issue in more details, if
                available.
        """

        class Severity(proto.Enum):
            r"""The severity of the issue.

            Values:
                SEVERITY_UNSPECIFIED (0):
                    Severity unspecified.
                WARNING (1):
                    The issue is the warning.
                ERROR (2):
                    The issue is an error.
            """
            SEVERITY_UNSPECIFIED = 0
            WARNING = 1
            ERROR = 2

        title: str = proto.Field(
            proto.STRING,
            number=1,
        )
        description: str = proto.Field(
            proto.STRING,
            number=2,
        )
        code: str = proto.Field(
            proto.STRING,
            number=3,
        )
        count: int = proto.Field(
            proto.INT64,
            number=4,
        )
        severity: "FileUpload.Issue.Severity" = proto.Field(
            proto.ENUM,
            number=5,
            enum="FileUpload.Issue.Severity",
        )
        documentation_uri: str = proto.Field(
            proto.STRING,
            number=6,
        )

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    data_source_id: int = proto.Field(
        proto.INT64,
        number=2,
    )
    processing_state: ProcessingState = proto.Field(
        proto.ENUM,
        number=3,
        enum=ProcessingState,
    )
    issues: MutableSequence[Issue] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message=Issue,
    )
    items_total: int = proto.Field(
        proto.INT64,
        number=5,
    )
    items_created: int = proto.Field(
        proto.INT64,
        number=6,
    )
    items_updated: int = proto.Field(
        proto.INT64,
        number=7,
    )
    upload_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=8,
        message=timestamp_pb2.Timestamp,
    )


class GetFileUploadRequest(proto.Message):
    r"""Request message for the GetFileUploadRequest method.

    Attributes:
        name (str):
            Required. The name of the data source file upload to
            retrieve. Format:
            ``accounts/{account}/dataSources/{datasource}/fileUploads/latest``
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os

# try/except added for compatibility with python < 3.8
try:
    from unittest import mock
    from unittest.mock import AsyncMock  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    import mock

from collections.abc import AsyncIterable, Iterable
import json
import math

from google.api_core import api_core_version
from google.protobuf import json_format
import grpc
from grpc.experimental import aio
from proto.marshal.rules import wrappers
from proto.marshal.rules.dates import DurationRule, TimestampRule
import pytest
from requests import PreparedRequest, Request, Response
from requests.sessions import Session

try:
    from google.auth.aio import credentials as ga_credentials_async

    HAS_GOOGLE_AUTH_AIO = True
except ImportError:  # pragma: NO COVER
    HAS_GOOGLE_AUTH_AIO = False

from google.api_core import gapic_v1, grpc_helpers, grpc_helpers_async, path_template
from google.api_core import client_options
from google.api_core import exceptions as core_exceptions
from google.api_core import retry as retries
import google.auth
from google.auth import credentials as ga_credentials
from google.auth.exceptions import MutualTLSChannelError
from google.oauth2 import service_account
from google.protobuf import timestamp_pb2  # type: ignore

from google.shopping.merchant_datasources_v1.services.file_uploads_service import (
    FileUploadsServiceAsyncClient,
    FileUploadsServiceClient,
    transports,
)
from google.shopping.merchant_datasources_v1.types import fileuploads

CRED_INFO_JSON = {
    "credential_source": "/path/to/file",
    "credential_type": "service account credentials",
    "principal": "service-account@example.com",
}
CRED_INFO_STRING = json.dumps(CRED_INFO_JSON)


async def mock_async_gen(data, chunk_size=1):
    for i in range(0, len(data)):  # pragma: NO COVER
        chunk = data[i : i + chunk_size]
        yield chunk.encode("utf-8")


def client_cert_source_callback():
    return b"cert bytes", b"key bytes"


# TODO: use async auth anon credentials by default once the minimum version of google-auth is upgraded.
# See related issue: https://github.com/googleapis/gapic-generator-python/issues/2107.
def async_anonymous_credentials():
    if HAS_GOOGLE_AUTH_AIO:
        return ga_credentials_async.AnonymousCredentials()
    return ga_credentials.AnonymousCredentials()


# If default endpoint is localhost, then default mtls endpoint will be the same.
# This method modifies the default endpoint so the client can produce a different
# mtls endpoint for endpoint testing purposes.
def modify_default_endpoint(client):
    return (
        "foo.googleapis.com"
        if ("localhost" in client.DEFAULT_ENDPOINT)
        else client.DEFAULT_ENDPOINT
    )


# If default endpoint template is localhost, then default mtls endpoint will be the same.
# This method modifies the default endpoint template so the client can produce a different
# mtls endpoint for endpoint testing purposes.
def modify_default_endpoint_template(client):
    return (
        "test.{UNIVERSE_DOMAIN}"
        if ("localhost" in client._DEFAULT_ENDPOINT_TEMPLATE)
        else client._DEFAULT_ENDPOINT_TEMPLATE
    )


def test__get_default_mtls_endpoint():
    api_endpoint = "example.googleapis.com"
    api_mtls_endpoint = "example.mtls.googleapis.com"
    sandbox_endpoint = "example.sandbox.googleapis.com"
    sandbox_mtls_endpoint = "example.mtls.sandbox.googleapis.com"
    non_googleapi = "api.example.com"

    assert FileUploadsServiceClient._get_default_mtls_endpoint(None) is None
    assert (
        FileUploadsServiceClient._get_default_mtls_endpoint(api_endpoint)
        == api_mtls_endpoint
    )
    assert (
        FileUploadsServiceClient._get_default_mtls_endpoint(api_mtls_endpoint)
        == api_mtls_endpoint
    )
    assert (
        FileUploadsServiceClient._get_default_mtls_endpoint(sandbox_endpoint)
        == sandbox_mtls_endpoint
    )
    assert (
        FileUploadsServiceClient._get_default_mtls_endpoint(sandbox_mtls_endpoint)
        == sandbox_mtls_endpoint
    )
    assert (
        FileUploadsServiceClient._get_default_mtls_endpoint(non_googleapi)
        == non_googleapi
    )


def test__read_environment_variables():
    assert FileUploadsServiceClient._read_environment_variables() == (
        False,
        "auto",
        None,
    )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            True,
            "auto",
            None,
        )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            False,
            "auto",
            None,
        )

    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            FileUploadsServiceClient._read_environment_variables()
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
    )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            False,
            "never",
            None,
        )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            False,
            "always",
            None,
        )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "auto"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            False,
            "auto",
            None,
        )

    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            FileUploadsServiceClient._read_environment_variables()
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
    )

    with mock.patch.dict(os.environ, {"GOOGLE_CLOUD_UNIVERSE_DOMAIN": "foo.com"}):
        assert FileUploadsServiceClient._read_environment_variables() == (
            False,
            "auto",
            "foo.com",
        )


def test__get_client_cert_source():
    mock_provided_cert_source = mock.Mock()
    mock_default_cert_source = mock.Mock()

    assert FileUploadsServiceClient._get_client_cert_source(None, False) is None
    assert (
        FileUploadsServiceClient._get_client_cert_source(
            mock_provided_cert_source, False
        )
        is None
    )
    assert (
        FileUploadsServiceClient._get_client_cert_source(
            mock_provided_cert_source, True
        )
        == mock_provided_cert_source
    )

    with mock.patch(
        "google.auth.transport.mtls.has_default_client_cert_source", return_value=True
    ):
        with mock.patch(
            "google.auth.transport.mtls.default_client_cert_source",
            return_value=mock_default_cert_source,
        ):
            assert (
                FileUploadsServiceClient._get_client_cert_source(None, True)
                is mock_default_cert_source
            )
            assert (
                FileUploadsServiceClient._get_client_cert_source(
                    mock_provided_cert_source, "true"
                )
                is mock_provided_cert_source
            )


@mock.patch.object(
    FileUploadsServiceClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceClient),
)
@mock.patch.object(
    FileUploadsServiceAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceAsyncClient),
)
def test__get_api_endpoint():
    api_override = "foo.com"
    mock_client_cert_source = mock.Mock()
    default_universe = FileUploadsServiceClient._DEFAULT_UNIVERSE
    default_endpoint = FileUploadsServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=default_universe
    )
    mock_universe = "bar.com"
    mock_endpoint = FileUploadsServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=mock_universe
    )

    assert (
        FileUploadsServiceClient._get_api_endpoint(
            api_override, mock_client_cert_source, default_universe, "always"
        )
        == api_override
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(
            None, mock_client_cert_source, default_universe, "auto"
        )
        == FileUploadsServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(None, None, default_universe, "auto")
        == default_endpoint
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(
            None, None, default_universe, "always"
        )
        == FileUploadsServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(
            None, mock_client_cert_source, default_universe, "always"
        )
        == FileUploadsServiceClient.DEFAULT_MTLS_ENDPOINT
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(None, None, mock_universe, "never")
        == mock_endpoint
    )
    assert (
        FileUploadsServiceClient._get_api_endpoint(
            None, None, default_universe, "never"
        )
        == default_endpoint
    )

    with pytest.raises(MutualTLSChannelError) as excinfo:
        FileUploadsServiceClient._get_api_endpoint(
            None, mock_client_cert_source, mock_universe, "auto"
        )
    assert (
        str(excinfo.value)
        == "mTLS is not supported in any universe other than googleapis.com."
    )


def test__get_universe_domain():
    client_universe_domain = "foo.com"
    universe_domain_env = "bar.com"

    assert (
        FileUploadsServiceClient._get_universe_domain(
            client_universe_domain, universe_domain_env
        )
        == client_universe_domain
    )
    assert (
        FileUploadsServiceClient._get_universe_domain(None, universe_domain_env)
        == universe_domain_env
    )
    assert (
        FileUploadsServiceClient._get_universe_domain(None, None)
        == FileUploadsServiceClient._DEFAULT_UNIVERSE
    )

    with pytest.raises(ValueError) as excinfo:
        FileUploadsServiceClient._get_universe_domain("", None)
    assert str(excinfo.value) == "Universe Domain cannot be an empty string."


@pytest.mark.parametrize(
    "error_code,cred_info_json,show_cred_info",
    [
        (401, CRED_INFO_JSON, True),
        (403, CRED_INFO_JSON, True),
        (404, CRED_INFO_JSON, True),
        (500, CRED_INFO_JSON, False),
        (401, None, False),
        (403, None, False),
        (404, None, False),
        (500, None, False),
    ],
)
def test__add_cred_info_for_auth_errors(error_code, cred_info_json, show_cred_info):
    cred = mock.Mock(["get_cred_info"])
    cred.get_cred_info = mock.Mock(return_value=cred_info_json)
    client = FileUploadsServiceClient(credentials=cred)
    client._transport._credentials = cred

    error = core_exceptions.GoogleAPICallError("message", details=["foo"])
    error.code = error_code

    client._add_cred_info_for_auth_errors(error)
    if show_cred_info:
        assert error.details == ["foo", CRED_INFO_STRING]
    else:
        assert error.details == ["foo"]


@pytest.mark.parametrize("error_code", [401, 403, 404, 500])
def test__add_cred_info_for_auth_errors_no_get_cred_info(error_code):
    cred = mock.Mock([])
    assert not hasattr(cred, "get_cred_info")
    client = FileUploadsServiceClient(credentials=cred)
    client._transport._credentials = cred

    error = core_exceptions.GoogleAPICallError("message", details=[])
    error.code = error_code

    client._add_cred_info_for_auth_errors(error)
    assert error.details == []


@pytest.mark.parametrize(
    "client_class,transport_name",
    [
        (FileUploadsServiceClient, "grpc"),
        (FileUploadsServiceAsyncClient, "grpc_asyncio"),
        (FileUploadsServiceClient, "rest"),
    ],
)
def test_file_uploads_service_client_from_service_account_info(
    client_class, transport_name
):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_info"
    ) as factory:
        factory.return_value = creds
        info = {"valid": True}
        client = client_class.from_service_account_info(info, transport=transport_name)
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == (
            "merchantapi.googleapis.com:443"
            if transport_name in ["grpc", "grpc_asyncio"]
            else "https://merchantapi.googleapis.com"
        )


@pytest.mark.parametrize(
    "transport_class,transport_name",
    [
        (transports.FileUploadsServiceGrpcTransport, "grpc"),
        (transports.FileUploadsServiceGrpcAsyncIOTransport, "grpc_asyncio"),
        (transports.FileUploadsServiceRestTransport, "rest"),
    ],
)
def test_file_uploads_service_client_service_account_always_use_jwt(
    transport_class, transport_name
):
    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=True)
        use_jwt.assert_called_once_with(True)

    with mock.patch.object(
        service_account.Credentials, "with_always_use_jwt_access", create=True
    ) as use_jwt:
        creds = service_account.Credentials(None, None, None)
        transport = transport_class(credentials=creds, always_use_jwt_access=False)
        use_jwt.assert_not_called()


@pytest.mark.parametrize(
    "client_class,transport_name",
    [
        (FileUploadsServiceClient, "grpc"),
        (FileUploadsServiceAsyncClient, "grpc_asyncio"),
        (FileUploadsServiceClient, "rest"),
    ],
)
def test_file_uploads_service_client_from_service_account_file(
    client_class, transport_name
):
    creds = ga_credentials.AnonymousCredentials()
    with mock.patch.object(
        service_account.Credentials, "from_service_account_file"
    ) as factory:
        factory.return_value = creds
        client = client_class.from_service_account_file(
            "dummy/file/path.json", transport=transport_name
        )
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        client = client_class.from_service_account_json(
            "dummy/file/path.json", transport=transport_name
        )
        assert client.transport._credentials == creds
        assert isinstance(client, client_class)

        assert client.transport._host == (
            "merchantapi.googleapis.com:443"
            if transport_name in ["grpc", "grpc_asyncio"]
            else "https://merchantapi.googleapis.com"
        )


def test_file_uploads_service_client_get_transport_class():
    transport = FileUploadsServiceClient.get_transport_class()
    available_transports = [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceRestTransport,
    ]
    assert transport in available_transports

    transport = FileUploadsServiceClient.get_transport_class("grpc")
    assert transport == transports.FileUploadsServiceGrpcTransport


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (FileUploadsServiceClient, transports.FileUploadsServiceGrpcTransport, "grpc"),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
        ),
        (FileUploadsServiceClient, transports.FileUploadsServiceRestTransport, "rest"),
    ],
)
@mock.patch.object(
    FileUploadsServiceClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceClient),
)
@mock.patch.object(
    FileUploadsServiceAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceAsyncClient),
)
def test_file_uploads_service_client_client_options(
    client_class, transport_class, transport_name
):
    # Check that if channel is provided we won't create a new one.
    with mock.patch.object(FileUploadsServiceClient, "get_transport_class") as gtc:
        transport = transport_class(credentials=ga_credentials.AnonymousCredentials())
        client = client_class(transport=transport)
        gtc.assert_not_called()

    # Check that if channel is provided via str we will create a new one.
    with mock.patch.object(FileUploadsServiceClient, "get_transport_class") as gtc:
        client = client_class(transport=transport_name)
        gtc.assert_called()

    # Check the case api_endpoint is provided.
    options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(transport=transport_name, client_options=options)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "never".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(transport=transport_name)
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                ),
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT is
    # "always".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(transport=transport_name)
            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=client.DEFAULT_MTLS_ENDPOINT,
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
    # unsupported value.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            client = client_class(transport=transport_name)
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
    )

    # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            client = client_class(transport=transport_name)
    assert (
        str(excinfo.value)
        == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
    )

    # Check the case quota_project_id is provided
    options = client_options.ClientOptions(quota_project_id="octopus")
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id="octopus",
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )
    # Check the case api_endpoint is provided
    options = client_options.ClientOptions(
        api_audience="https://language.googleapis.com"
    )
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience="https://language.googleapis.com",
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,use_client_cert_env",
    [
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceGrpcTransport,
            "grpc",
            "true",
        ),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
            "true",
        ),
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceGrpcTransport,
            "grpc",
            "false",
        ),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
            "false",
        ),
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceRestTransport,
            "rest",
            "true",
        ),
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceRestTransport,
            "rest",
            "false",
        ),
    ],
)
@mock.patch.object(
    FileUploadsServiceClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceClient),
)
@mock.patch.object(
    FileUploadsServiceAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceAsyncClient),
)
@mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "auto"})
def test_file_uploads_service_client_mtls_env_auto(
    client_class, transport_class, transport_name, use_client_cert_env
):
    # This tests the endpoint autoswitch behavior. Endpoint is autoswitched to the default
    # mtls endpoint, if GOOGLE_API_USE_CLIENT_CERTIFICATE is "true" and client cert exists.

    # Check the case client_cert_source is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        options = client_options.ClientOptions(
            client_cert_source=client_cert_source_callback
        )
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(client_options=options, transport=transport_name)

            if use_client_cert_env == "false":
                expected_client_cert_source = None
                expected_host = client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                )
            else:
                expected_client_cert_source = client_cert_source_callback
                expected_host = client.DEFAULT_MTLS_ENDPOINT

            patched.assert_called_once_with(
                credentials=None,
                credentials_file=None,
                host=expected_host,
                scopes=None,
                client_cert_source_for_mtls=expected_client_cert_source,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )

    # Check the case ADC client cert is provided. Whether client cert is used depends on
    # GOOGLE_API_USE_CLIENT_CERTIFICATE value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=True,
            ):
                with mock.patch(
                    "google.auth.transport.mtls.default_client_cert_source",
                    return_value=client_cert_source_callback,
                ):
                    if use_client_cert_env == "false":
                        expected_host = client._DEFAULT_ENDPOINT_TEMPLATE.format(
                            UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                        )
                        expected_client_cert_source = None
                    else:
                        expected_host = client.DEFAULT_MTLS_ENDPOINT
                        expected_client_cert_source = client_cert_source_callback

                    patched.return_value = None
                    client = client_class(transport=transport_name)
                    patched.assert_called_once_with(
                        credentials=None,
                        credentials_file=None,
                        host=expected_host,
                        scopes=None,
                        client_cert_source_for_mtls=expected_client_cert_source,
                        quota_project_id=None,
                        client_info=transports.base.DEFAULT_CLIENT_INFO,
                        always_use_jwt_access=True,
                        api_audience=None,
                    )

    # Check the case client_cert_source and ADC client cert are not provided.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": use_client_cert_env}
    ):
        with mock.patch.object(transport_class, "__init__") as patched:
            with mock.patch(
                "google.auth.transport.mtls.has_default_client_cert_source",
                return_value=False,
            ):
                patched.return_value = None
                client = client_class(transport=transport_name)
                patched.assert_called_once_with(
                    credentials=None,
                    credentials_file=None,
                    host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                        UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                    ),
                    scopes=None,
                    client_cert_source_for_mtls=None,
                    quota_project_id=None,
                    client_info=transports.base.DEFAULT_CLIENT_INFO,
                    always_use_jwt_access=True,
                    api_audience=None,
                )


@pytest.mark.parametrize(
    "client_class", [FileUploadsServiceClient, FileUploadsServiceAsyncClient]
)
@mock.patch.object(
    FileUploadsServiceClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(FileUploadsServiceClient),
)
@mock.patch.object(
    FileUploadsServiceAsyncClient,
    "DEFAULT_ENDPOINT",
    modify_default_endpoint(FileUploadsServiceAsyncClient),
)
def test_file_uploads_service_client_get_mtls_endpoint_and_cert_source(client_class):
    mock_client_cert_source = mock.Mock()

    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        mock_api_endpoint = "foo"
        options = client_options.ClientOptions(
            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
        )
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
            options
        )
        assert api_endpoint == mock_api_endpoint
        assert cert_source == mock_client_cert_source

    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
        mock_client_cert_source = mock.Mock()
        mock_api_endpoint = "foo"
        options = client_options.ClientOptions(
            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
        )
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
            options
        )
        assert api_endpoint == mock_api_endpoint
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
        assert api_endpoint == client_class.DEFAULT_ENDPOINT
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
        assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.mtls.has_default_client_cert_source",
            return_value=False,
        ):
            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
            assert api_endpoint == client_class.DEFAULT_ENDPOINT
            assert cert_source is None

    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.mtls.has_default_client_cert_source",
            return_value=True,
        ):
            with mock.patch(
                "google.auth.transport.mtls.default_client_cert_source",
                return_value=mock_client_cert_source,
            ):
                (
                    api_endpoint,
                    cert_source,
                ) = client_class.get_mtls_endpoint_and_cert_source()
                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
                assert cert_source == mock_client_cert_source

    # Check the case api_endpoint is not provided and GOOGLE_API_USE_MTLS_ENDPOINT has
    # unsupported value.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
        with pytest.raises(MutualTLSChannelError) as excinfo:
            client_class.get_mtls_endpoint_and_cert_source()

        assert (
            str(excinfo.value)
            == "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
        )

    # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
    with mock.patch.dict(
        os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
    ):
        with pytest.raises(ValueError) as excinfo:
            client_class.get_mtls_endpoint_and_cert_source()

        assert (
            str(excinfo.value)
            == "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
        )


@pytest.mark.parametrize(
    "client_class", [FileUploadsServiceClient, FileUploadsServiceAsyncClient]
)
@mock.patch.object(
    FileUploadsServiceClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceClient),
)
@mock.patch.object(
    FileUploadsServiceAsyncClient,
    "_DEFAULT_ENDPOINT_TEMPLATE",
    modify_default_endpoint_template(FileUploadsServiceAsyncClient),
)
def test_file_uploads_service_client_client_api_endpoint(client_class):
    mock_client_cert_source = client_cert_source_callback
    api_override = "foo.com"
    default_universe = FileUploadsServiceClient._DEFAULT_UNIVERSE
    default_endpoint = FileUploadsServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=default_universe
    )
    mock_universe = "bar.com"
    mock_endpoint = FileUploadsServiceClient._DEFAULT_ENDPOINT_TEMPLATE.format(
        UNIVERSE_DOMAIN=mock_universe
    )

    # If ClientOptions.api_endpoint is set and GOOGLE_API_USE_CLIENT_CERTIFICATE="true",
    # use ClientOptions.api_endpoint as the api endpoint regardless.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
        with mock.patch(
            "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
        ):
            options = client_options.ClientOptions(
                client_cert_source=mock_client_cert_source, api_endpoint=api_override
            )
            client = client_class(
                client_options=options,
                credentials=ga_credentials.AnonymousCredentials(),
            )
            assert client.api_endpoint == api_override

    # If ClientOptions.api_endpoint is not set and GOOGLE_API_USE_MTLS_ENDPOINT="never",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with GDU as the api endpoint.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        client = client_class(credentials=ga_credentials.AnonymousCredentials())
        assert client.api_endpoint == default_endpoint

    # If ClientOptions.api_endpoint is not set and GOOGLE_API_USE_MTLS_ENDPOINT="always",
    # use the DEFAULT_MTLS_ENDPOINT as the api endpoint.
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
        client = client_class(credentials=ga_credentials.AnonymousCredentials())
        assert client.api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT

    # If ClientOptions.api_endpoint is not set, GOOGLE_API_USE_MTLS_ENDPOINT="auto" (default),
    # GOOGLE_API_USE_CLIENT_CERTIFICATE="false" (default), default cert source doesn't exist,
    # and ClientOptions.universe_domain="bar.com",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with universe domain as the api endpoint.
    options = client_options.ClientOptions()
    universe_exists = hasattr(options, "universe_domain")
    if universe_exists:
        options = client_options.ClientOptions(universe_domain=mock_universe)
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
    else:
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
    assert client.api_endpoint == (
        mock_endpoint if universe_exists else default_endpoint
    )
    assert client.universe_domain == (
        mock_universe if universe_exists else default_universe
    )

    # If ClientOptions does not have a universe domain attribute and GOOGLE_API_USE_MTLS_ENDPOINT="never",
    # use the _DEFAULT_ENDPOINT_TEMPLATE populated with GDU as the api endpoint.
    options = client_options.ClientOptions()
    if hasattr(options, "universe_domain"):
        delattr(options, "universe_domain")
    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
        client = client_class(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )
        assert client.api_endpoint == default_endpoint


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name",
    [
        (FileUploadsServiceClient, transports.FileUploadsServiceGrpcTransport, "grpc"),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
        ),
        (FileUploadsServiceClient, transports.FileUploadsServiceRestTransport, "rest"),
    ],
)
def test_file_uploads_service_client_client_options_scopes(
    client_class, transport_class, transport_name
):
    # Check the case scopes are provided.
    options = client_options.ClientOptions(
        scopes=["1", "2"],
    )
    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=["1", "2"],
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,grpc_helpers",
    [
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceGrpcTransport,
            "grpc",
            grpc_helpers,
        ),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
            grpc_helpers_async,
        ),
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceRestTransport,
            "rest",
            None,
        ),
    ],
)
def test_file_uploads_service_client_client_options_credentials_file(
    client_class, transport_class, transport_name, grpc_helpers
):
    # Check the case credentials file is provided.
    options = client_options.ClientOptions(credentials_file="credentials.json")

    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file="credentials.json",
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


def test_file_uploads_service_client_client_options_from_dict():
    with mock.patch(
        "google.shopping.merchant_datasources_v1.services.file_uploads_service.transports.FileUploadsServiceGrpcTransport.__init__"
    ) as grpc_transport:
        grpc_transport.return_value = None
        client = FileUploadsServiceClient(
            client_options={"api_endpoint": "squid.clam.whelk"}
        )
        grpc_transport.assert_called_once_with(
            credentials=None,
            credentials_file=None,
            host="squid.clam.whelk",
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )


@pytest.mark.parametrize(
    "client_class,transport_class,transport_name,grpc_helpers",
    [
        (
            FileUploadsServiceClient,
            transports.FileUploadsServiceGrpcTransport,
            "grpc",
            grpc_helpers,
        ),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
            "grpc_asyncio",
            grpc_helpers_async,
        ),
    ],
)
def test_file_uploads_service_client_create_channel_credentials_file(
    client_class, transport_class, transport_name, grpc_helpers
):
    # Check the case credentials file is provided.
    options = client_options.ClientOptions(credentials_file="credentials.json")

    with mock.patch.object(transport_class, "__init__") as patched:
        patched.return_value = None
        client = client_class(client_options=options, transport=transport_name)
        patched.assert_called_once_with(
            credentials=None,
            credentials_file="credentials.json",
            host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
            ),
            scopes=None,
            client_cert_source_for_mtls=None,
            quota_project_id=None,
            client_info=transports.base.DEFAULT_CLIENT_INFO,
            always_use_jwt_access=True,
            api_audience=None,
        )

    # test that the credentials from file are saved and used as the credentials.
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch.object(
        google.auth, "default", autospec=True
    ) as adc, mock.patch.object(
        grpc_helpers, "create_channel"
    ) as create_channel:
        creds = ga_credentials.AnonymousCredentials()
        file_creds = ga_credentials.AnonymousCredentials()
        load_creds.return_value = (file_creds, None)
        adc.return_value = (creds, None)
        client = client_class(client_options=options, transport=transport_name)
        create_channel.assert_called_with(
            "merchantapi.googleapis.com:443",
            credentials=file_creds,
            credentials_file=None,
            quota_project_id=None,
            default_scopes=("https://www.googleapis.com/auth/content",),
            scopes=None,
            default_host="merchantapi.googleapis.com",
            ssl_credentials=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )


@pytest.mark.parametrize(
    "request_type",
    [
        fileuploads.GetFileUploadRequest,
        dict,
    ],
)
def test_get_file_upload(request_type, transport: str = "grpc"):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = fileuploads.FileUpload(
            name="name_value",
            data_source_id=1462,
            processing_state=fileuploads.FileUpload.ProcessingState.FAILED,
            items_total=1189,
            items_created=1369,
            items_updated=1384,
        )
        response = client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        request = fileuploads.GetFileUploadRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, fileuploads.FileUpload)
    assert response.name == "name_value"
    assert response.data_source_id == 1462
    assert response.processing_state == fileuploads.FileUpload.ProcessingState.FAILED
    assert response.items_total == 1189
    assert response.items_created == 1369
    assert response.items_updated == 1384


def test_get_file_upload_non_empty_request_with_auto_populated_field():
    # This test is a coverage failsafe to make sure that UUID4 fields are
    # automatically populated, according to AIP-4235, with non-empty requests.
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Populate all string fields in the request which are not UUID4
    # since we want to check that UUID4 are populated automatically
    # if they meet the requirements of AIP 4235.
    request = fileuploads.GetFileUploadRequest(
        name="name_value",
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        call.return_value.name = (
            "foo"  # operation_request.operation in compute client(s) expect a string.
        )
        client.get_file_upload(request=request)
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        assert args[0] == fileuploads.GetFileUploadRequest(
            name="name_value",
        )


def test_get_file_upload_use_cached_wrapped_rpc():
    # Clients should use _prep_wrapped_messages to create cached wrapped rpcs,
    # instead of constructing them on each call
    with mock.patch("google.api_core.gapic_v1.method.wrap_method") as wrapper_fn:
        client = FileUploadsServiceClient(
            credentials=ga_credentials.AnonymousCredentials(),
            transport="grpc",
        )

        # Should wrap all calls on client creation
        assert wrapper_fn.call_count > 0
        wrapper_fn.reset_mock()

        # Ensure method has been cached
        assert client._transport.get_file_upload in client._transport._wrapped_methods

        # Replace cached wrapped function with mock
        mock_rpc = mock.Mock()
        mock_rpc.return_value.name = (
            "foo"  # operation_request.operation in compute client(s) expect a string.
        )
        client._transport._wrapped_methods[client._transport.get_file_upload] = mock_rpc
        request = {}
        client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert mock_rpc.call_count == 1

        client.get_file_upload(request)

        # Establish that a new wrapper was not created for this call
        assert wrapper_fn.call_count == 0
        assert mock_rpc.call_count == 2


@pytest.mark.asyncio
async def test_get_file_upload_async_use_cached_wrapped_rpc(
    transport: str = "grpc_asyncio",
):
    # Clients should use _prep_wrapped_messages to create cached wrapped rpcs,
    # instead of constructing them on each call
    with mock.patch("google.api_core.gapic_v1.method_async.wrap_method") as wrapper_fn:
        client = FileUploadsServiceAsyncClient(
            credentials=async_anonymous_credentials(),
            transport=transport,
        )

        # Should wrap all calls on client creation
        assert wrapper_fn.call_count > 0
        wrapper_fn.reset_mock()

        # Ensure method has been cached
        assert (
            client._client._transport.get_file_upload
            in client._client._transport._wrapped_methods
        )

        # Replace cached wrapped function with mock
        mock_rpc = mock.AsyncMock()
        mock_rpc.return_value = mock.Mock()
        client._client._transport._wrapped_methods[
            client._client._transport.get_file_upload
        ] = mock_rpc

        request = {}
        await client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert mock_rpc.call_count == 1

        await client.get_file_upload(request)

        # Establish that a new wrapper was not created for this call
        assert wrapper_fn.call_count == 0
        assert mock_rpc.call_count == 2


@pytest.mark.asyncio
async def test_get_file_upload_async(
    transport: str = "grpc_asyncio", request_type=fileuploads.GetFileUploadRequest
):
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(),
        transport=transport,
    )

    # Everything is optional in proto3 as far as the runtime is concerned,
    # and we are mocking out the actual API, so just send an empty request.
    request = request_type()

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            fileuploads.FileUpload(
                name="name_value",
                data_source_id=1462,
                processing_state=fileuploads.FileUpload.ProcessingState.FAILED,
                items_total=1189,
                items_created=1369,
                items_updated=1384,
            )
        )
        response = await client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        request = fileuploads.GetFileUploadRequest()
        assert args[0] == request

    # Establish that the response is the type that we expect.
    assert isinstance(response, fileuploads.FileUpload)
    assert response.name == "name_value"
    assert response.data_source_id == 1462
    assert response.processing_state == fileuploads.FileUpload.ProcessingState.FAILED
    assert response.items_total == 1189
    assert response.items_created == 1369
    assert response.items_updated == 1384


@pytest.mark.asyncio
async def test_get_file_upload_async_from_dict():
    await test_get_file_upload_async(request_type=dict)


def test_get_file_upload_field_headers():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = fileuploads.GetFileUploadRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        call.return_value = fileuploads.FileUpload()
        client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


@pytest.mark.asyncio
async def test_get_file_upload_field_headers_async():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(),
    )

    # Any value that is part of the HTTP/1.1 URI should be sent as
    # a field header. Set these to a non-empty value.
    request = fileuploads.GetFileUploadRequest()

    request.name = "name_value"

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            fileuploads.FileUpload()
        )
        await client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        assert args[0] == request

    # Establish that the field header was sent.
    _, _, kw = call.mock_calls[0]
    assert (
        "x-goog-request-params",
        "name=name_value",
    ) in kw["metadata"]


def test_get_file_upload_flattened():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = fileuploads.FileUpload()
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        client.get_file_upload(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls) == 1
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


def test_get_file_upload_flattened_error():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_file_upload(
            fileuploads.GetFileUploadRequest(),
            name="name_value",
        )


@pytest.mark.asyncio
async def test_get_file_upload_flattened_async():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(),
    )

    # Mock the actual call within the gRPC stub, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = fileuploads.FileUpload()

        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            fileuploads.FileUpload()
        )
        # Call the method with a truthy value for each flattened field,
        # using the keyword arguments to the method.
        response = await client.get_file_upload(
            name="name_value",
        )

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(call.mock_calls)
        _, args, _ = call.mock_calls[0]
        arg = args[0].name
        mock_val = "name_value"
        assert arg == mock_val


@pytest.mark.asyncio
async def test_get_file_upload_flattened_error_async():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(),
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        await client.get_file_upload(
            fileuploads.GetFileUploadRequest(),
            name="name_value",
        )


def test_get_file_upload_rest_use_cached_wrapped_rpc():
    # Clients should use _prep_wrapped_messages to create cached wrapped rpcs,
    # instead of constructing them on each call
    with mock.patch("google.api_core.gapic_v1.method.wrap_method") as wrapper_fn:
        client = FileUploadsServiceClient(
            credentials=ga_credentials.AnonymousCredentials(),
            transport="rest",
        )

        # Should wrap all calls on client creation
        assert wrapper_fn.call_count > 0
        wrapper_fn.reset_mock()

        # Ensure method has been cached
        assert client._transport.get_file_upload in client._transport._wrapped_methods

        # Replace cached wrapped function with mock
        mock_rpc = mock.Mock()
        mock_rpc.return_value.name = (
            "foo"  # operation_request.operation in compute client(s) expect a string.
        )
        client._transport._wrapped_methods[client._transport.get_file_upload] = mock_rpc

        request = {}
        client.get_file_upload(request)

        # Establish that the underlying gRPC stub method was called.
        assert mock_rpc.call_count == 1

        client.get_file_upload(request)

        # Establish that a new wrapper was not created for this call
        assert wrapper_fn.call_count == 0
        assert mock_rpc.call_count == 2


def test_get_file_upload_rest_required_fields(
    request_type=fileuploads.GetFileUploadRequest,
):
    transport_class = transports.FileUploadsServiceRestTransport

    request_init = {}
    request_init["name"] = ""
    request = request_type(**request_init)
    pb_request = request_type.pb(request)
    jsonified_request = json.loads(
        json_format.MessageToJson(pb_request, use_integers_for_enums=False)
    )

    # verify fields with default values are dropped

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_file_upload._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with default values are now present

    jsonified_request["name"] = "name_value"

    unset_fields = transport_class(
        credentials=ga_credentials.AnonymousCredentials()
    ).get_file_upload._get_unset_required_fields(jsonified_request)
    jsonified_request.update(unset_fields)

    # verify required fields with non-default values are left alone
    assert "name" in jsonified_request
    assert jsonified_request["name"] == "name_value"

    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )
    request = request_type(**request_init)

    # Designate an appropriate value for the returned response.
    return_value = fileuploads.FileUpload()
    # Mock the http request call within the method and fake a response.
    with mock.patch.object(Session, "request") as req:
        # We need to mock transcode() because providing default values
        # for required fields will fail the real version if the http_options
        # expect actual values for those fields.
        with mock.patch.object(path_template, "transcode") as transcode:
            # A uri without fields and an empty body will force all the
            # request fields to show up in the query_params.
            pb_request = request_type.pb(request)
            transcode_result = {
                "uri": "v1/sample_method",
                "method": "get",
                "query_params": pb_request,
            }
            transcode.return_value = transcode_result

            response_value = Response()
            response_value.status_code = 200

            # Convert return value to protobuf type
            return_value = fileuploads.FileUpload.pb(return_value)
            json_return_value = json_format.MessageToJson(return_value)

            response_value._content = json_return_value.encode("UTF-8")
            req.return_value = response_value
            req.return_value.headers = {"header-1": "value-1", "header-2": "value-2"}

            response = client.get_file_upload(request)

            expected_params = [("$alt", "json;enum-encoding=int")]
            actual_params = req.call_args.kwargs["params"]
            assert expected_params == actual_params


def test_get_file_upload_rest_unset_required_fields():
    transport = transports.FileUploadsServiceRestTransport(
        credentials=ga_credentials.AnonymousCredentials
    )

    unset_fields = transport.get_file_upload._get_unset_required_fields({})
    assert set(unset_fields) == (set(()) & set(("name",)))


def test_get_file_upload_rest_flattened():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = fileuploads.FileUpload()

        # get arguments that satisfy an http rule for this method
        sample_request = {
            "name": "accounts/sample1/dataSources/sample2/fileUploads/sample3"
        }

        # get truthy value for each flattened field
        mock_args = dict(
            name="name_value",
        )
        mock_args.update(sample_request)

        # Wrap the value into a proper Response obj
        response_value = Response()
        response_value.status_code = 200
        # Convert return value to protobuf type
        return_value = fileuploads.FileUpload.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)
        response_value._content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        req.return_value.headers = {"header-1": "value-1", "header-2": "value-2"}

        client.get_file_upload(**mock_args)

        # Establish that the underlying call was made with the expected
        # request object values.
        assert len(req.mock_calls) == 1
        _, args, _ = req.mock_calls[0]
        assert path_template.validate(
            "%s/datasources/v1/{name=accounts/*/dataSources/*/fileUploads/*}"
            % client.transport._host,
            args[1],
        )


def test_get_file_upload_rest_flattened_error(transport: str = "rest"):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport=transport,
    )

    # Attempting to call a method with both a request object and flattened
    # fields is an error.
    with pytest.raises(ValueError):
        client.get_file_upload(
            fileuploads.GetFileUploadRequest(),
            name="name_value",
        )


def test_credentials_transport_error():
    # It is an error to provide credentials and a transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = FileUploadsServiceClient(
            credentials=ga_credentials.AnonymousCredentials(),
            transport=transport,
        )

    # It is an error to provide a credentials file and a transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = FileUploadsServiceClient(
            client_options={"credentials_file": "credentials.json"},
            transport=transport,
        )

    # It is an error to provide an api_key and a transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    options = client_options.ClientOptions()
    options.api_key = "api_key"
    with pytest.raises(ValueError):
        client = FileUploadsServiceClient(
            client_options=options,
            transport=transport,
        )

    # It is an error to provide an api_key and a credential.
    options = client_options.ClientOptions()
    options.api_key = "api_key"
    with pytest.raises(ValueError):
        client = FileUploadsServiceClient(
            client_options=options, credentials=ga_credentials.AnonymousCredentials()
        )

    # It is an error to provide scopes and a transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    with pytest.raises(ValueError):
        client = FileUploadsServiceClient(
            client_options={"scopes": ["1", "2"]},
            transport=transport,
        )


def test_transport_instance():
    # A client may be instantiated with a custom transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    client = FileUploadsServiceClient(transport=transport)
    assert client.transport is transport


def test_transport_get_channel():
    # A client may be instantiated with a custom transport instance.
    transport = transports.FileUploadsServiceGrpcTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel

    transport = transports.FileUploadsServiceGrpcAsyncIOTransport(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    channel = transport.grpc_channel
    assert channel


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
        transports.FileUploadsServiceRestTransport,
    ],
)
def test_transport_adc(transport_class):
    # Test default credentials are used if not provided.
    with mock.patch.object(google.auth, "default") as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class()
        adc.assert_called_once()


def test_transport_kind_grpc():
    transport = FileUploadsServiceClient.get_transport_class("grpc")(
        credentials=ga_credentials.AnonymousCredentials()
    )
    assert transport.kind == "grpc"


def test_initialize_client_w_grpc():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc"
    )
    assert client is not None


# This test is a coverage failsafe to make sure that totally empty calls,
# i.e. request == None and no flattened fields passed, work.
def test_get_file_upload_empty_call_grpc():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="grpc",
    )

    # Mock the actual call, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        call.return_value = fileuploads.FileUpload()
        client.get_file_upload(request=None)

        # Establish that the underlying stub method was called.
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        request_msg = fileuploads.GetFileUploadRequest()

        assert args[0] == request_msg


def test_transport_kind_grpc_asyncio():
    transport = FileUploadsServiceAsyncClient.get_transport_class("grpc_asyncio")(
        credentials=async_anonymous_credentials()
    )
    assert transport.kind == "grpc_asyncio"


def test_initialize_client_w_grpc_asyncio():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(), transport="grpc_asyncio"
    )
    assert client is not None


# This test is a coverage failsafe to make sure that totally empty calls,
# i.e. request == None and no flattened fields passed, work.
@pytest.mark.asyncio
async def test_get_file_upload_empty_call_grpc_asyncio():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(),
        transport="grpc_asyncio",
    )

    # Mock the actual call, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        # Designate an appropriate return value for the call.
        call.return_value = grpc_helpers_async.FakeUnaryUnaryCall(
            fileuploads.FileUpload(
                name="name_value",
                data_source_id=1462,
                processing_state=fileuploads.FileUpload.ProcessingState.FAILED,
                items_total=1189,
                items_created=1369,
                items_updated=1384,
            )
        )
        await client.get_file_upload(request=None)

        # Establish that the underlying stub method was called.
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        request_msg = fileuploads.GetFileUploadRequest()

        assert args[0] == request_msg


def test_transport_kind_rest():
    transport = FileUploadsServiceClient.get_transport_class("rest")(
        credentials=ga_credentials.AnonymousCredentials()
    )
    assert transport.kind == "rest"


def test_get_file_upload_rest_bad_request(
    request_type=fileuploads.GetFileUploadRequest,
):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )
    # send a request that will satisfy transcoding
    request_init = {"name": "accounts/sample1/dataSources/sample2/fileUploads/sample3"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a BadRequest error.
    with mock.patch.object(Session, "request") as req, pytest.raises(
        core_exceptions.BadRequest
    ):
        # Wrap the value into a proper Response obj
        response_value = mock.Mock()
        json_return_value = ""
        response_value.json = mock.Mock(return_value={})
        response_value.status_code = 400
        response_value.request = mock.Mock()
        req.return_value = response_value
        req.return_value.headers = {"header-1": "value-1", "header-2": "value-2"}
        client.get_file_upload(request)


@pytest.mark.parametrize(
    "request_type",
    [
        fileuploads.GetFileUploadRequest,
        dict,
    ],
)
def test_get_file_upload_rest_call_success(request_type):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )

    # send a request that will satisfy transcoding
    request_init = {"name": "accounts/sample1/dataSources/sample2/fileUploads/sample3"}
    request = request_type(**request_init)

    # Mock the http request call within the method and fake a response.
    with mock.patch.object(type(client.transport._session), "request") as req:
        # Designate an appropriate value for the returned response.
        return_value = fileuploads.FileUpload(
            name="name_value",
            data_source_id=1462,
            processing_state=fileuploads.FileUpload.ProcessingState.FAILED,
            items_total=1189,
            items_created=1369,
            items_updated=1384,
        )

        # Wrap the value into a proper Response obj
        response_value = mock.Mock()
        response_value.status_code = 200

        # Convert return value to protobuf type
        return_value = fileuploads.FileUpload.pb(return_value)
        json_return_value = json_format.MessageToJson(return_value)
        response_value.content = json_return_value.encode("UTF-8")
        req.return_value = response_value
        req.return_value.headers = {"header-1": "value-1", "header-2": "value-2"}
        response = client.get_file_upload(request)

    # Establish that the response is the type that we expect.
    assert isinstance(response, fileuploads.FileUpload)
    assert response.name == "name_value"
    assert response.data_source_id == 1462
    assert response.processing_state == fileuploads.FileUpload.ProcessingState.FAILED
    assert response.items_total == 1189
    assert response.items_created == 1369
    assert response.items_updated == 1384


@pytest.mark.parametrize("null_interceptor", [True, False])
def test_get_file_upload_rest_interceptors(null_interceptor):
    transport = transports.FileUploadsServiceRestTransport(
        credentials=ga_credentials.AnonymousCredentials(),
        interceptor=None
        if null_interceptor
        else transports.FileUploadsServiceRestInterceptor(),
    )
    client = FileUploadsServiceClient(transport=transport)

    with mock.patch.object(
        type(client.transport._session), "request"
    ) as req, mock.patch.object(
        path_template, "transcode"
    ) as transcode, mock.patch.object(
        transports.FileUploadsServiceRestInterceptor, "post_get_file_upload"
    ) as post, mock.patch.object(
        transports.FileUploadsServiceRestInterceptor,
        "post_get_file_upload_with_metadata",
    ) as post_with_metadata, mock.patch.object(
        transports.FileUploadsServiceRestInterceptor, "pre_get_file_upload"
    ) as pre:
        pre.assert_not_called()
        post.assert_not_called()
        post_with_metadata.assert_not_called()
        pb_message = fileuploads.GetFileUploadRequest.pb(
            fileuploads.GetFileUploadRequest()
        )
        transcode.return_value = {
            "method": "post",
            "uri": "my_uri",
            "body": pb_message,
            "query_params": pb_message,
        }

        req.return_value = mock.Mock()
        req.return_value.status_code = 200
        req.return_value.headers = {"header-1": "value-1", "header-2": "value-2"}
        return_value = fileuploads.FileUpload.to_json(fileuploads.FileUpload())
        req.return_value.content = return_value

        request = fileuploads.GetFileUploadRequest()
        metadata = [
            ("key", "val"),
            ("cephalopod", "squid"),
        ]
        pre.return_value = request, metadata
        post.return_value = fileuploads.FileUpload()
        post_with_metadata.return_value = fileuploads.FileUpload(), metadata

        client.get_file_upload(
            request,
            metadata=[
                ("key", "val"),
                ("cephalopod", "squid"),
            ],
        )

        pre.assert_called_once()
        post.assert_called_once()
        post_with_metadata.assert_called_once()


def test_initialize_client_w_rest():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )
    assert client is not None


# This test is a coverage failsafe to make sure that totally empty calls,
# i.e. request == None and no flattened fields passed, work.
def test_get_file_upload_empty_call_rest():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        transport="rest",
    )

    # Mock the actual call, and fake the request.
    with mock.patch.object(type(client.transport.get_file_upload), "__call__") as call:
        client.get_file_upload(request=None)

        # Establish that the underlying stub method was called.
        call.assert_called()
        _, args, _ = call.mock_calls[0]
        request_msg = fileuploads.GetFileUploadRequest()

        assert args[0] == request_msg


def test_transport_grpc_default():
    # A client should use the gRPC transport by default.
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
    )
    assert isinstance(
        client.transport,
        transports.FileUploadsServiceGrpcTransport,
    )


def test_file_uploads_service_base_transport_error():
    # Passing both a credentials object and credentials_file should raise an error
    with pytest.raises(core_exceptions.DuplicateCredentialArgs):
        transport = transports.FileUploadsServiceTransport(
            credentials=ga_credentials.AnonymousCredentials(),
            credentials_file="credentials.json",
        )


def test_file_uploads_service_base_transport():
    # Instantiate the base transport.
    with mock.patch(
        "google.shopping.merchant_datasources_v1.services.file_uploads_service.transports.FileUploadsServiceTransport.__init__"
    ) as Transport:
        Transport.return_value = None
        transport = transports.FileUploadsServiceTransport(
            credentials=ga_credentials.AnonymousCredentials(),
        )

    # Every method on the transport should just blindly
    # raise NotImplementedError.
    methods = ("get_file_upload",)
    for method in methods:
        with pytest.raises(NotImplementedError):
            getattr(transport, method)(request=object())

    with pytest.raises(NotImplementedError):
        transport.close()

    # Catch all for all remaining methods and properties
    remainder = [
        "kind",
    ]
    for r in remainder:
        with pytest.raises(NotImplementedError):
            getattr(transport, r)()


def test_file_uploads_service_base_transport_with_credentials_file():
    # Instantiate the base transport with a credentials file
    with mock.patch.object(
        google.auth, "load_credentials_from_file", autospec=True
    ) as load_creds, mock.patch(
        "google.shopping.merchant_datasources_v1.services.file_uploads_service.transports.FileUploadsServiceTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.FileUploadsServiceTransport(
            credentials_file="credentials.json",
            quota_project_id="octopus",
        )
        load_creds.assert_called_once_with(
            "credentials.json",
            scopes=None,
            default_scopes=("https://www.googleapis.com/auth/content",),
            quota_project_id="octopus",
        )


def test_file_uploads_service_base_transport_with_adc():
    # Test the default credentials are used if credentials and credentials_file are None.
    with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
        "google.shopping.merchant_datasources_v1.services.file_uploads_service.transports.FileUploadsServiceTransport._prep_wrapped_messages"
    ) as Transport:
        Transport.return_value = None
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport = transports.FileUploadsServiceTransport()
        adc.assert_called_once()


def test_file_uploads_service_auth_adc():
    # If no credentials are provided, we should use ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        FileUploadsServiceClient()
        adc.assert_called_once_with(
            scopes=None,
            default_scopes=("https://www.googleapis.com/auth/content",),
            quota_project_id=None,
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
    ],
)
def test_file_uploads_service_transport_auth_adc(transport_class):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(google.auth, "default", autospec=True) as adc:
        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])
        adc.assert_called_once_with(
            scopes=["1", "2"],
            default_scopes=("https://www.googleapis.com/auth/content",),
            quota_project_id="octopus",
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
        transports.FileUploadsServiceRestTransport,
    ],
)
def test_file_uploads_service_transport_auth_gdch_credentials(transport_class):
    host = "https://language.com"
    api_audience_tests = [None, "https://language2.com"]
    api_audience_expect = [host, "https://language2.com"]
    for t, e in zip(api_audience_tests, api_audience_expect):
        with mock.patch.object(google.auth, "default", autospec=True) as adc:
            gdch_mock = mock.MagicMock()
            type(gdch_mock).with_gdch_audience = mock.PropertyMock(
                return_value=gdch_mock
            )
            adc.return_value = (gdch_mock, None)
            transport_class(host=host, api_audience=t)
            gdch_mock.with_gdch_audience.assert_called_once_with(e)


@pytest.mark.parametrize(
    "transport_class,grpc_helpers",
    [
        (transports.FileUploadsServiceGrpcTransport, grpc_helpers),
        (transports.FileUploadsServiceGrpcAsyncIOTransport, grpc_helpers_async),
    ],
)
def test_file_uploads_service_transport_create_channel(transport_class, grpc_helpers):
    # If credentials and host are not provided, the transport class should use
    # ADC credentials.
    with mock.patch.object(
        google.auth, "default", autospec=True
    ) as adc, mock.patch.object(
        grpc_helpers, "create_channel", autospec=True
    ) as create_channel:
        creds = ga_credentials.AnonymousCredentials()
        adc.return_value = (creds, None)
        transport_class(quota_project_id="octopus", scopes=["1", "2"])

        create_channel.assert_called_with(
            "merchantapi.googleapis.com:443",
            credentials=creds,
            credentials_file=None,
            quota_project_id="octopus",
            default_scopes=("https://www.googleapis.com/auth/content",),
            scopes=["1", "2"],
            default_host="merchantapi.googleapis.com",
            ssl_credentials=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )


@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
    ],
)
def test_file_uploads_service_grpc_transport_client_cert_source_for_mtls(
    transport_class,
):
    cred = ga_credentials.AnonymousCredentials()

    # Check ssl_channel_credentials is used if provided.
    with mock.patch.object(transport_class, "create_channel") as mock_create_channel:
        mock_ssl_channel_creds = mock.Mock()
        transport_class(
            host="squid.clam.whelk",
            credentials=cred,
            ssl_channel_credentials=mock_ssl_channel_creds,
        )
        mock_create_channel.assert_called_once_with(
            "squid.clam.whelk:443",
            credentials=cred,
            credentials_file=None,
            scopes=None,
            ssl_credentials=mock_ssl_channel_creds,
            quota_project_id=None,
            options=[
                ("grpc.max_send_message_length", -1),
                ("grpc.max_receive_message_length", -1),
            ],
        )

    # Check if ssl_channel_credentials is not provided, then client_cert_source_for_mtls
    # is used.
    with mock.patch.object(transport_class, "create_channel", return_value=mock.Mock()):
        with mock.patch("grpc.ssl_channel_credentials") as mock_ssl_cred:
            transport_class(
                credentials=cred,
                client_cert_source_for_mtls=client_cert_source_callback,
            )
            expected_cert, expected_key = client_cert_source_callback()
            mock_ssl_cred.assert_called_once_with(
                certificate_chain=expected_cert, private_key=expected_key
            )


def test_file_uploads_service_http_transport_client_cert_source_for_mtls():
    cred = ga_credentials.AnonymousCredentials()
    with mock.patch(
        "google.auth.transport.requests.AuthorizedSession.configure_mtls_channel"
    ) as mock_configure_mtls_channel:
        transports.FileUploadsServiceRestTransport(
            credentials=cred, client_cert_source_for_mtls=client_cert_source_callback
        )
        mock_configure_mtls_channel.assert_called_once_with(client_cert_source_callback)


@pytest.mark.parametrize(
    "transport_name",
    [
        "grpc",
        "grpc_asyncio",
        "rest",
    ],
)
def test_file_uploads_service_host_no_port(transport_name):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="merchantapi.googleapis.com"
        ),
        transport=transport_name,
    )
    assert client.transport._host == (
        "merchantapi.googleapis.com:443"
        if transport_name in ["grpc", "grpc_asyncio"]
        else "https://merchantapi.googleapis.com"
    )


@pytest.mark.parametrize(
    "transport_name",
    [
        "grpc",
        "grpc_asyncio",
        "rest",
    ],
)
def test_file_uploads_service_host_with_port(transport_name):
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(),
        client_options=client_options.ClientOptions(
            api_endpoint="merchantapi.googleapis.com:8000"
        ),
        transport=transport_name,
    )
    assert client.transport._host == (
        "merchantapi.googleapis.com:8000"
        if transport_name in ["grpc", "grpc_asyncio"]
        else "https://merchantapi.googleapis.com:8000"
    )


@pytest.mark.parametrize(
    "transport_name",
    [
        "rest",
    ],
)
def test_file_uploads_service_client_transport_session_collision(transport_name):
    creds1 = ga_credentials.AnonymousCredentials()
    creds2 = ga_credentials.AnonymousCredentials()
    client1 = FileUploadsServiceClient(
        credentials=creds1,
        transport=transport_name,
    )
    client2 = FileUploadsServiceClient(
        credentials=creds2,
        transport=transport_name,
    )
    session1 = client1.transport.get_file_upload._session
    session2 = client2.transport.get_file_upload._session
    assert session1 != session2


def test_file_uploads_service_grpc_transport_channel():
    channel = grpc.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.FileUploadsServiceGrpcTransport(
        host="squid.clam.whelk",
        channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


def test_file_uploads_service_grpc_asyncio_transport_channel():
    channel = aio.secure_channel("http://localhost/", grpc.local_channel_credentials())

    # Check that channel is used if provided.
    transport = transports.FileUploadsServiceGrpcAsyncIOTransport(
        host="squid.clam.whelk",
        channel=channel,
    )
    assert transport.grpc_channel == channel
    assert transport._host == "squid.clam.whelk:443"
    assert transport._ssl_channel_credentials == None


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
    ],
)
def test_file_uploads_service_transport_channel_mtls_with_client_cert_source(
    transport_class,
):
    with mock.patch(
        "grpc.ssl_channel_credentials", autospec=True
    ) as grpc_ssl_channel_cred:
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_ssl_cred = mock.Mock()
            grpc_ssl_channel_cred.return_value = mock_ssl_cred

            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel

            cred = ga_credentials.AnonymousCredentials()
            with pytest.warns(DeprecationWarning):
                with mock.patch.object(google.auth, "default") as adc:
                    adc.return_value = (cred, None)
                    transport = transport_class(
                        host="squid.clam.whelk",
                        api_mtls_endpoint="mtls.squid.clam.whelk",
                        client_cert_source=client_cert_source_callback,
                    )
                    adc.assert_called_once()

            grpc_ssl_channel_cred.assert_called_once_with(
                certificate_chain=b"cert bytes", private_key=b"key bytes"
            )
            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel
            assert transport._ssl_channel_credentials == mock_ssl_cred


# Remove this test when deprecated arguments (api_mtls_endpoint, client_cert_source) are
# removed from grpc/grpc_asyncio transport constructor.
@pytest.mark.parametrize(
    "transport_class",
    [
        transports.FileUploadsServiceGrpcTransport,
        transports.FileUploadsServiceGrpcAsyncIOTransport,
    ],
)
def test_file_uploads_service_transport_channel_mtls_with_adc(transport_class):
    mock_ssl_cred = mock.Mock()
    with mock.patch.multiple(
        "google.auth.transport.grpc.SslCredentials",
        __init__=mock.Mock(return_value=None),
        ssl_credentials=mock.PropertyMock(return_value=mock_ssl_cred),
    ):
        with mock.patch.object(
            transport_class, "create_channel"
        ) as grpc_create_channel:
            mock_grpc_channel = mock.Mock()
            grpc_create_channel.return_value = mock_grpc_channel
            mock_cred = mock.Mock()

            with pytest.warns(DeprecationWarning):
                transport = transport_class(
                    host="squid.clam.whelk",
                    credentials=mock_cred,
                    api_mtls_endpoint="mtls.squid.clam.whelk",
                    client_cert_source=None,
                )

            grpc_create_channel.assert_called_once_with(
                "mtls.squid.clam.whelk:443",
                credentials=mock_cred,
                credentials_file=None,
                scopes=None,
                ssl_credentials=mock_ssl_cred,
                quota_project_id=None,
                options=[
                    ("grpc.max_send_message_length", -1),
                    ("grpc.max_receive_message_length", -1),
                ],
            )
            assert transport.grpc_channel == mock_grpc_channel


def test_file_upload_path():
    account = "squid"
    datasource = "clam"
    fileupload = "whelk"
    expected = (
        "accounts/{account}/dataSources/{datasource}/fileUploads/{fileupload}".format(
            account=account,
            datasource=datasource,
            fileupload=fileupload,
        )
    )
    actual = FileUploadsServiceClient.file_upload_path(account, datasource, fileupload)
    assert expected == actual


def test_parse_file_upload_path():
    expected = {
        "account": "octopus",
        "datasource": "oyster",
        "fileupload": "nudibranch",
    }
    path = FileUploadsServiceClient.file_upload_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_file_upload_path(path)
    assert expected == actual


def test_common_billing_account_path():
    billing_account = "cuttlefish"
    expected = "billingAccounts/{billing_account}".format(
        billing_account=billing_account,
    )
    actual = FileUploadsServiceClient.common_billing_account_path(billing_account)
    assert expected == actual


def test_parse_common_billing_account_path():
    expected = {
        "billing_account": "mussel",
    }
    path = FileUploadsServiceClient.common_billing_account_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_common_billing_account_path(path)
    assert expected == actual


def test_common_folder_path():
    folder = "winkle"
    expected = "folders/{folder}".format(
        folder=folder,
    )
    actual = FileUploadsServiceClient.common_folder_path(folder)
    assert expected == actual


def test_parse_common_folder_path():
    expected = {
        "folder": "nautilus",
    }
    path = FileUploadsServiceClient.common_folder_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_common_folder_path(path)
    assert expected == actual


def test_common_organization_path():
    organization = "scallop"
    expected = "organizations/{organization}".format(
        organization=organization,
    )
    actual = FileUploadsServiceClient.common_organization_path(organization)
    assert expected == actual


def test_parse_common_organization_path():
    expected = {
        "organization": "abalone",
    }
    path = FileUploadsServiceClient.common_organization_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_common_organization_path(path)
    assert expected == actual


def test_common_project_path():
    project = "squid"
    expected = "projects/{project}".format(
        project=project,
    )
    actual = FileUploadsServiceClient.common_project_path(project)
    assert expected == actual


def test_parse_common_project_path():
    expected = {
        "project": "clam",
    }
    path = FileUploadsServiceClient.common_project_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_common_project_path(path)
    assert expected == actual


def test_common_location_path():
    project = "whelk"
    location = "octopus"
    expected = "projects/{project}/locations/{location}".format(
        project=project,
        location=location,
    )
    actual = FileUploadsServiceClient.common_location_path(project, location)
    assert expected == actual


def test_parse_common_location_path():
    expected = {
        "project": "oyster",
        "location": "nudibranch",
    }
    path = FileUploadsServiceClient.common_location_path(**expected)

    # Check that the path construction is reversible.
    actual = FileUploadsServiceClient.parse_common_location_path(path)
    assert expected == actual


def test_client_with_default_client_info():
    client_info = gapic_v1.client_info.ClientInfo()

    with mock.patch.object(
        transports.FileUploadsServiceTransport, "_prep_wrapped_messages"
    ) as prep:
        client = FileUploadsServiceClient(
            credentials=ga_credentials.AnonymousCredentials(),
            client_info=client_info,
        )
        prep.assert_called_once_with(client_info)

    with mock.patch.object(
        transports.FileUploadsServiceTransport, "_prep_wrapped_messages"
    ) as prep:
        transport_class = FileUploadsServiceClient.get_transport_class()
        transport = transport_class(
            credentials=ga_credentials.AnonymousCredentials(),
            client_info=client_info,
        )
        prep.assert_called_once_with(client_info)


def test_transport_close_grpc():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="grpc"
    )
    with mock.patch.object(
        type(getattr(client.transport, "_grpc_channel")), "close"
    ) as close:
        with client:
            close.assert_not_called()
        close.assert_called_once()


@pytest.mark.asyncio
async def test_transport_close_grpc_asyncio():
    client = FileUploadsServiceAsyncClient(
        credentials=async_anonymous_credentials(), transport="grpc_asyncio"
    )
    with mock.patch.object(
        type(getattr(client.transport, "_grpc_channel")), "close"
    ) as close:
        async with client:
            close.assert_not_called()
        close.assert_called_once()


def test_transport_close_rest():
    client = FileUploadsServiceClient(
        credentials=ga_credentials.AnonymousCredentials(), transport="rest"
    )
    with mock.patch.object(
        type(getattr(client.transport, "_session")), "close"
    ) as close:
        with client:
            close.assert_not_called()
        close.assert_called_once()


def test_client_ctx():
    transports = [
        "rest",
        "grpc",
    ]
    for transport in transports:
        client = FileUploadsServiceClient(
            credentials=ga_credentials.AnonymousCredentials(), transport=transport
        )
        # Test client calls underlying transport.
        with mock.patch.object(type(client.transport), "close") as close:
            close.assert_not_called()
            with client:
                pass
            close.assert_called()


@pytest.mark.parametrize(
    "client_class,transport_class",
    [
        (FileUploadsServiceClient, transports.FileUploadsServiceGrpcTransport),
        (
            FileUploadsServiceAsyncClient,
            transports.FileUploadsServiceGrpcAsyncIOTransport,
        ),
    ],
)
def test_api_key_credentials(client_class, transport_class):
    with mock.patch.object(
        google.auth._default, "get_api_key_credentials", create=True
    ) as get_api_key_credentials:
        mock_cred = mock.Mock()
        get_api_key_credentials.return_value = mock_cred
        options = client_options.ClientOptions()
        options.api_key = "api_key"
        with mock.patch.object(transport_class, "__init__") as patched:
            patched.return_value = None
            client = client_class(client_options=options)
            patched.assert_called_once_with(
                credentials=mock_cred,
                credentials_file=None,
                host=client._DEFAULT_ENDPOINT_TEMPLATE.format(
                    UNIVERSE_DOMAIN=client._DEFAULT_UNIVERSE
                ),
                scopes=None,
                client_cert_source_for_mtls=None,
                quota_project_id=None,
                client_info=transports.base.DEFAULT_CLIENT_INFO,
                always_use_jwt_access=True,
                api_audience=None,
            )



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.type import dayofweek_pb2  # type: ignore
from google.type import timeofday_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.shopping.merchant.datasources.v1beta",
    manifest={
        "FileInput",
    },
)


class FileInput(proto.Message):
    r"""The data specific for file data sources. This field is empty
    for other data source inputs.

    Attributes:
        fetch_settings (google.shopping.merchant_datasources_v1beta.types.FileInput.FetchSettings):
            Optional. Fetch details to deliver the data source. It
            contains settings for ``FETCH`` and ``GOOGLE_SHEETS`` file
            input types. The required fields vary based on the frequency
            of fetching.
        file_name (str):
            Optional. The file name of the data source. Required for
            ``UPLOAD`` file input type.
        file_input_type (google.shopping.merchant_datasources_v1beta.types.FileInput.FileInputType):
            Output only. The type of file input.
    """

    class FileInputType(proto.Enum):
        r"""The method of file delivery.

        Values:
            FILE_INPUT_TYPE_UNSPECIFIED (0):
                File input type unspecified.
            UPLOAD (1):
                The file is uploaded through SFTP, Google
                Cloud Storage or manually in the Merchant
                Center.
            FETCH (2):
                The file is fetched from the configured
                [fetch_uri][google.shopping.merchant.datasources.v1beta.FileInput.FetchSettings.fetch_uri].
            GOOGLE_SHEETS (3):
                The file is fetched from Google Sheets specified in the
                [fetch_uri][google.shopping.merchant.datasources.v1beta.FileInput.FetchSettings.fetch_uri].
        """
        FILE_INPUT_TYPE_UNSPECIFIED = 0
        UPLOAD = 1
        FETCH = 2
        GOOGLE_SHEETS = 3

    class FetchSettings(proto.Message):
        r"""Fetch details to deliver the data source.

        Attributes:
            enabled (bool):
                Optional. Enables or pauses the fetch
                schedule.
            day_of_month (int):
                Optional. The day of the month when the data
                source file should be fetched (1-31). This field
                can only be set for monthly frequency.
            time_of_day (google.type.timeofday_pb2.TimeOfDay):
                Optional. The hour of the day when the data
                source file should be fetched. Minutes and
                seconds are not supported and will be ignored.
            day_of_week (google.type.dayofweek_pb2.DayOfWeek):
                Optional. The day of the week when the data
                source file should be fetched. This field can
                only be set for weekly frequency.
            time_zone (str):
                Optional. `Time zone <https://cldr.unicode.org>`__ used for
                schedule. UTC by default. For example,
                "America/Los_Angeles".
            frequency (google.shopping.merchant_datasources_v1beta.types.FileInput.FetchSettings.Frequency):
                Required. The frequency describing fetch
                schedule.
            fetch_uri (str):
                Optional. The URL where the data source file
                can be fetched. Google Merchant Center supports
                automatic scheduled uploads using the HTTP,
                HTTPS or SFTP protocols, so the value will need
                to be a valid link using one of those three
                protocols. Immutable for Google Sheets files.
            username (str):
                Optional. An optional user name for
                [fetch_uri][google.shopping.merchant.datasources.v1beta.FileInput.FetchSettings.fetch_uri].
                Used for `submitting data sources through
                SFTP <https://support.google.com/merchants/answer/13813117>`__.
            password (str):
                Optional. An optional password for
                [fetch_uri][google.shopping.merchant.datasources.v1beta.FileInput.FetchSettings.fetch_uri].
                Used for `submitting data sources through
                SFTP <https://support.google.com/merchants/answer/13813117>`__.
        """

        class Frequency(proto.Enum):
            r"""The required fields vary based on the frequency of fetching. For a
            monthly fetch schedule, [day of
            month][google.shopping.content.bundles.DataSources.FileInput.FetchSchedule.day_of_month]
            and [hour of
            day][google.shopping.content.bundles.DataSources.FileInput.FetchSchedule.time_of_day]
            are required. For a weekly fetch schedule, [day of
            week][google.shopping.content.bundles.DataSources.FileInput.FetchSchedule.day_of_week]
            and [hour of
            day][google.shopping.content.bundles.DataSources.FileInput.FetchSchedule.time_of_day]
            are required. For a daily fetch schedule, only an [hour of
            day][google.shopping.content.bundles.DataSources.FileInput.FetchSchedule.time_of_day]
            is required.

            Values:
                FREQUENCY_UNSPECIFIED (0):
                    Frequency unspecified.
                FREQUENCY_DAILY (1):
                    The fetch happens every day.
                FREQUENCY_WEEKLY (2):
                    The fetch happens every week.
                FREQUENCY_MONTHLY (3):
                    The fetch happens every month.
            """
            FREQUENCY_UNSPECIFIED = 0
            FREQUENCY_DAILY = 1
            FREQUENCY_WEEKLY = 2
            FREQUENCY_MONTHLY = 3

        enabled: bool = proto.Field(
            proto.BOOL,
            number=1,
        )
        day_of_month: int = proto.Field(
            proto.INT32,
            number=2,
        )
        time_of_day: timeofday_pb2.TimeOfDay = proto.Field(
            proto.MESSAGE,
            number=3,
            message=timeofday_pb2.TimeOfDay,
        )
        day_of_week: dayofweek_pb2.DayOfWeek = proto.Field(
            proto.ENUM,
            number=4,
            enum=dayofweek_pb2.DayOfWeek,
        )
        time_zone: str = proto.Field(
            proto.STRING,
            number=5,
        )
        frequency: "FileInput.FetchSettings.Frequency" = proto.Field(
            proto.ENUM,
            number=6,
            enum="FileInput.FetchSettings.Frequency",
        )
        fetch_uri: str = proto.Field(
            proto.STRING,
            number=7,
        )
        username: str = proto.Field(
            proto.STRING,
            number=8,
        )
        password: str = proto.Field(
            proto.STRING,
            number=9,
        )

    fetch_settings: FetchSettings = proto.Field(
        proto.MESSAGE,
        number=1,
        message=FetchSettings,
    )
    file_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    file_input_type: FileInputType = proto.Field(
        proto.ENUM,
        number=3,
        enum=FileInputType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.shopping.merchant_datasources import gapic_version as package_version

__version__ = package_version.__version__


from google.shopping.merchant_datasources_v1.services.data_sources_service.async_client import (
    DataSourcesServiceAsyncClient,
)
from google.shopping.merchant_datasources_v1.services.data_sources_service.client import (
    DataSourcesServiceClient,
)
from google.shopping.merchant_datasources_v1.services.file_uploads_service.async_client import (
    FileUploadsServiceAsyncClient,
)
from google.shopping.merchant_datasources_v1.services.file_uploads_service.client import (
    FileUploadsServiceClient,
)
from google.shopping.merchant_datasources_v1.types.datasources import (
    CreateDataSourceRequest,
    DataSource,
    DeleteDataSourceRequest,
    FetchDataSourceRequest,
    GetDataSourceRequest,
    ListDataSourcesRequest,
    ListDataSourcesResponse,
    UpdateDataSourceRequest,
)
from google.shopping.merchant_datasources_v1.types.datasourcetypes import (
    DataSourceReference,
    LocalInventoryDataSource,
    MerchantReviewDataSource,
    PrimaryProductDataSource,
    ProductReviewDataSource,
    PromotionDataSource,
    RegionalInventoryDataSource,
    SupplementalProductDataSource,
)
from google.shopping.merchant_datasources_v1.types.fileinputs import FileInput
from google.shopping.merchant_datasources_v1.types.fileuploads import (
    FileUpload,
    GetFileUploadRequest,
)

__all__ = (
    "DataSourcesServiceClient",
    "DataSourcesServiceAsyncClient",
    "FileUploadsServiceClient",
    "FileUploadsServiceAsyncClient",
    "CreateDataSourceRequest",
    "DataSource",
    "DeleteDataSourceRequest",
    "FetchDataSourceRequest",
    "GetDataSourceRequest",
    "ListDataSourcesRequest",
    "ListDataSourcesResponse",
    "UpdateDataSourceRequest",
    "DataSourceReference",
    "LocalInventoryDataSource",
    "MerchantReviewDataSource",
    "PrimaryProductDataSource",
    "ProductReviewDataSource",
    "PromotionDataSource",
    "RegionalInventoryDataSource",
    "SupplementalProductDataSource",
    "FileInput",
    "FileUpload",
    "GetFileUploadRequest",
)
